@@ -40,12 +40,14 @@ SERVICE_RENAMES = {
     'elb': 'elastic-load-balancing',
     'elbv2': 'elastic-load-balancing-v2',
     'es': 'elasticsearch-service',
-    'events': 'cloudwatch-events',
+    'events': 'eventbridge',
     'globalaccelerator': 'global-accelerator',
     'iot-data': 'iot-data-plane',
     'iot-jobs-data': 'iot-jobs-data-plane',
     'iot1click-devices': 'iot-1click-devices-service',
     'iot1click-projects': 'iot-1click-projects',
+    'iotevents-data': 'iot-events-data',
+    'iotevents': 'iot-events',
     'kinesisanalytics': 'kinesis-analytics',
     'kinesisanalyticsv2': 'kinesis-analytics-v2',
     'kinesisvideo': 'kinesis-video',
@@ -81,6 +83,8 @@ ENDPOINT_PREFIX_OVERRIDE = {
     # For neptune, we send requests to the RDS endpoint.
     'neptune': 'rds',
     'docdb': 'rds',
+    # iotevents data endpoints.json and service-2.json don't line up.
+    'ioteventsdata': 'data.iotevents',
 }
 NOT_SUPPORTED_IN_SDK = [
     'mobileanalytics',
